// (C) Copyright StyleTap Inc. 2005. All Rights Reserved

// SampleNativeLibraryCall.cpp

#include <PalmOS.h>
#include <TextMgr.h>
#include "StyleTapPlatform.h"


// this code fragment illustrates how to use the Native Library support

static inline unsigned32 flip (unsigned32 x)
{
    return (x << 24) | ((x & 0xff00) << 8) | 
           ((x & 0x00ff0000) >> 8) | (x >> 24);
}

//////////////////////////////////////////////////////


static Int16 maxLines;

#define top			20
#define lineHeight	15

// following class implements a simple wrapping display of messages
// on the form above the button

static class Display
{
	Int16 line;

public:
	Display () {line = 0;}

    virtual void printf (const char *formatStr, ...) 
	{
		Char s[500];
		va_list args;
		va_start(args, formatStr);
		StrVPrintF (s, formatStr, args);
		va_end (args);

		RectangleType rect;
		rect.topLeft.x = 2; 
		rect.extent.x = 156;
		rect.extent.y = lineHeight; 

		Int16 len = StrLen (s);
		Char *seg = s;
		for (;;)
		{

			Int16 width = FntCharsWidth (seg, len);
			if (len == 0)
				break;
			else if (width <= 156)
			{
				rect.topLeft.y = top + (line * lineHeight);
				WinEraseRectangle (&rect, 0);
				WinDrawChars (seg, len, rect.topLeft.x, rect.topLeft.y);

				seg += len;
				len = StrLen (seg);
				line = (line + 1) % maxLines;
			}
			else
				len--;
		}
	}
} display;

// the code for the client follows

static void test ()
{
    StpErr rc = 0;
    unsigned32 libraryHandle = 0;

    rc = LoadNativeLibrary ("SampleNativeLibrary.dll", &libraryHandle);
    if (rc == StpOK)
    {
		typedef int (*FuncAddParmsType)(signed32 a, signed32 b, signed32 c, 
									    signed32 d, signed32 e);
		FuncAddParmsType FuncAddParms = NULL;
		rc = GetNativeLibraryFunctionAddressByOrdinal (libraryHandle,
								       				   1,       
								       				   (void**) &FuncAddParms);
        if (rc == StpOK)
        {
            unsigned32 returnValue = 0;
            int a = 1, b = 2, c = 4, d = 8, e = 16;
            unsigned32 params [] = 
                {
                    flip((unsigned32) a),
                    flip((unsigned32) b),
                    flip((unsigned32) c),
                    flip((unsigned32) d),
                    flip((unsigned32) e),
                };
            
        	rc = CallNativeLibraryFunction (FuncAddParms, 
        									params, 
        									sizeof(params), 
        									&returnValue, 
        									NULL);

            if (rc == StpOK)
            {
                display.printf ("FuncAddParms() returned: %lu", returnValue);
            }
            else
                display.printf ("CallNativeLibraryFunction() returned: %lu", rc);
        }
        else
            display.printf ("GetNativeLibraryFunctionAddressByOrdinal() returned: %lu", rc);
        
		rc = UnloadNativeLibrary (libraryHandle); 
		if (rc != StpOK)
			display.printf ("UnloadNativeLibrary() returned: %lu", rc);
    }
    else
        display.printf ("LoadNativeLibrary() returned: %lu", rc);
}

////////////////////////////////////////////

// The following is a little more generalized than needed
// for this particular sample program. The idea is that
// multiple test routines can be added, and the following
// code will arrange buttons at the bottom of the form dynamically.
// Tapping the button triggers the test routine to be 
// called. In this case we have only one routine!

typedef void (*Handler) ();

static struct
{
    Handler handler;
    Char    *title;
}
handlers [] = 
{
// list test routines in this array:
    {test, "Call FuncAddParms"},
};

/////

static bool isRunningOnStyleTapForSymbian ()
{
    bool result = false;

    UInt32 companyID;
    UInt32 deviceID;
    UInt32 versionBuildNumber;
    Err companyErr;
    Err deviceErr;
    Err versionErr;

    companyErr = FtrGet (sysFtrCreator, sysFtrNumOEMCompanyID,
                         &companyID);

    deviceErr = FtrGet (sysFtrCreator, sysFtrNumOEMDeviceID,
                        &deviceID);

    if ((companyErr == 0) && (companyID == 'stap') 
    	&& (deviceErr == 0) && (deviceID == 'symb')
    	)
    {
        // This is StyleTap!
        Err versionError = FtrGet (sysFtrCreator,
                                   sysFtrNumOEMROMVersion,
                                   &versionBuildNumber);
        result = true;
    }
    return result;
}



#define ARRAYSIZE(a) (sizeof(a)/sizeof(a[0]))
#define BUTTON_ID       2000
#define BUTTON_WIDTH    30
#define BUTTON_HEIGHT   15

static Boolean FormEventHandler (EventPtr event)
{     
    FormType  *form;
    int       handled = 0;
    
    switch (event->eType) 
    {
    case ctlSelectEvent:
        (*handlers [event->data.ctlSelect.controlID - BUTTON_ID].handler) ();
        handled = 1;
        break;
    default:
        break;
    }
    return handled;
}

static void prepForm ()
{
    FormType *f = FrmNewForm (1000, //id
                              "Native Library Call Test", // title
                              0, 0, 160, 160, // x,y,width,height
                              false, //modal
                              BUTTON_ID, //default button
                              0, //helprsc
                              0); // menursc
    if (f != NULL)
    {
        FntSetFont (stdFont);
        Int16 x = 5;
        Int16 y = 140;
        for (unsigned i = 0; i < ARRAYSIZE(handlers); i++)
        {
            Int16 buttonWidth = FntCharsWidth (handlers [i].title, StrLen(handlers [i].title)) + 5;

            if ((x + buttonWidth) > 158)
            {
                x = 5;
                y -= BUTTON_HEIGHT + 5;
            }

            ControlType *b = CtlNewControl ((void**)&f, 
                                            BUTTON_ID + i, 
                                            buttonCtl, 
                                            handlers [i].title, 
                                            x, 
                                            y,
                                            buttonWidth,
                                            BUTTON_HEIGHT,
                                            stdFont, 
                                            0, //group
                                            false //leftAnchor
                                            );
            x += buttonWidth + 5;
        }
        maxLines = (y - top - 2) / lineHeight;

        FrmSetActiveForm (f);
        FrmDrawForm (f);
    }
}

static void NormalLaunch ()
{
	if (isRunningOnStyleTapForSymbian ())
	{
		unsigned short err;
		EventType event;
	
		prepForm ();
	
		do
		{
			EvtGetEvent (&event, 0);
			
			if (!SysHandleEvent (&event) &&
				!MenuHandleEvent ((MenuBarPtr)0, &event, &err) &&
				!FormEventHandler (&event))
			{
				FrmDispatchEvent (&event);
			}
		} 
		while (event.eType != appStopEvent);
	
		FrmEraseForm (FrmGetActiveForm ());
		FrmDeleteForm (FrmGetActiveForm ());
	}
	else
	{
		// not running on StyleTap CrossPlatform for Symbian: bail silently
	}
}

unsigned long PilotMain (unsigned short cmd, void *cmdPBP, unsigned short launchFlags)
{
    if (cmd == sysAppLaunchCmdNormalLaunch)
    {
        NormalLaunch ();
    }
    return 0;
}
