// Copyright (c) 2005-2008 StyleTap Inc. All Rights Reserved.

// StyleTap Platform API
// Version 1.1.012

#ifndef StyleTapPlatform_h
#define StyleTapPlatform_h

// basic type definitions

typedef unsigned long  unsigned32;
typedef signed long    signed32;
typedef unsigned short unsigned16;
typedef signed short   signed16;
typedef unsigned char  unsigned8;
typedef signed char    signed8;

#define StyleTapCompanyID  'stap'

// custom vfs api selectors
#define OBJECT_STORE_SLOT_REF 0xff00
#define OBJECT_STORE_VOL_REF  0xff00


#define StyleTapFileFlush       1

// screen position when SIP comes up
typedef unsigned8 SipScreenPositionType;

#define defaultPosition    0
#define dontMove           1

#define ledSpecialEffect     0
#define vibrateSpecialEffect 1

// notifications

#if defined(_WIN32_WCE) || defined(WIN32)
#pragma pack(push, 1) // structures are now packed
#endif

struct NotifyWndMessage 
{
    UInt16 msg;
    UInt16 wParam;
    UInt32 lParam;
};

#if defined(_WIN32_WCE) || defined(WIN32)
#pragma pack(pop) // packing returns to previous state
#endif

#define NotifyWindowsMessage 'stwm'
#define WM_STYLETAP_APP  0x8100 // starting Windows message available to StyleTap app (WM_APP + 0x100)
#define WM_STYLETAP_APP_MAX (WM_STYLETAP_APP + 0x2000)

// error codes:

typedef unsigned long  StpErr;

#define StpOK			            0
#define StpUnimplemented            0xffffffff
#define StpDesktopNotConnected      0x0001
#define StpParamError               0x0002
#define StpUnspecifiedError         0x0003
#define StpFileNotFound             0x0004
#define StpBufferTooSmall           0x0005
#define StpOutOfMemory              0x0006
#define StpRegistered               0x0007
#define StpTrial                    0x0008
#define StpNoConnection             0x0009
#define StpInvalidAccountIdPassword 0x000a
#define StpInsufficientCredit       0x000b
#define StpInvalidProductKey        0x000c
#define StpTimeout                  0x000d
#define StpHttpProtocolError        0x000e
#define StpBadDataFromHost          0x000f
#define StpDbNotFound               0x0010
#define StpRecordDbRequired         0x0011
#define StpDbOpen                   0x0012
#define StpLockError                0x0013
#define StpUserCancel               0x0014
#define StpWriteDenied              0x0204
#define StpNotSupportedByDevice     0x0015 
#define StpCharEncodingUnavailable  0x0016
#define StpPermissionDenied         0x0017

// trap interface definitions
#define StyleTapTrap15			                0x1000
#define GetDesktopIpAddressRoutine              1
#define LoadNativeLibraryRoutine                2
#define GetNativeLibraryFunctionAddressRoutine  3
#define CallNativeLibraryFunctionRoutine        4
#define UnloadNativeLibraryRoutine              5
#define BringToForegroundRoutine                6
#define StpAcceptRegistrationKeyRoutine         7
#define StpGetProductKeyRoutine                 8
#define DatabaseFlushRoutine                    9
#define DatabaseFlushAllRoutine                 10
#define GetMainWindowHandleRoutine              11
#define StpPPPConnectRoutine			        12
#define StpPPPHangUpRoutine			            13
#define StpPPPConfigRoutine                     14
#define StpExitRoutine					        15
#define StpGetRegistrationStateRoutine          16
#define StpGetRegistrationKeyFromServerRoutine  17 // deprecated
#define StpRefreshDisplayWindowRoutine          18
#define StpSetSerialPortMappingRoutine          20
#define StpGetSerialPortMappingRoutine          21

#define StpPostDatabaseRoutineObsolete             		29
#define StpGetDatabaseRoutineObsolete              		30
#define StpSetFastScreenModeRoutine                		31
#define StpSetSipScreenPositionRoutine             		32
#define StpUnloadDatabaseRoutine                   		33
#define StpUnloadDatabasesRoutine                  		34
#define StpPostDatabaseRoutine                     		35
#define StpGetDatabaseRoutine                      		36
#define StpUpdateDisplayBufferRoutine              		37
#define StpSpecialEffectRoutine                    		38 
#define StpGetRegistrationKeyFromServerRoutineExt  		39
#define StpGetObexReceiverInstallStatusRoutine     		40
#define StpSetObexReceiverInstallStatusRoutine     		41
#define StpIsCharEncodingAvailableRoutine          		42
#define StpSetCurrentCharEncodingRoutine           		43
#define StpGetCurrentCharEncodingRoutine           		44
#define GetNativeLibraryFunctionAddressByOrdinalRoutine 45
#define StpSetSimulateDoubleDensityScreenRoutine        46
#define StpGetSimulateDoubleDensityScreenRoutine        47

#if defined(__GNUC__) && !defined(__SYMBIAN32__)
#define lit(s) #s
#define STYLETAP_TRAP(ROUTINE) __attribute__ ((__callseq__ ( \
		"moveq #" lit(ROUTINE) ",%%d2; trap #15; dc.w 0x1000")))
#elif defined(__MWERKS__)
#define STYLETAP_TRAP(ROUTINE) = { 0x7400 + ROUTINE, 0x4e4f, StyleTapTrap15 }
#else
#define STYLETAP_TRAP(ROUTINE) 
#endif

// functions

#ifdef __cplusplus
extern "C" {
#endif

extern StpErr GetDesktopIpAddress (unsigned8 IpAddr [4]) // returned
				      STYLETAP_TRAP(GetDesktopIpAddressRoutine);

extern StpErr LoadNativeLibrary (const char *libraryFileName, // supplied
                                 unsigned32 *libraryHandle)   // returned
                      STYLETAP_TRAP(LoadNativeLibraryRoutine);

extern StpErr GetNativeLibraryFunctionAddress (unsigned32 handle,         // supplied
                                               const char *functionName,  // supplied
                                               void       **functionAddr) // returned
                      STYLETAP_TRAP(GetNativeLibraryFunctionAddressRoutine);


extern StpErr GetNativeLibraryFunctionAddressByOrdinal (unsigned32 handle,  	   // supplied
										       			unsigned32 ordinal,        // supplied
										       			void       **functionAddr) // returned
                      STYLETAP_TRAP(GetNativeLibraryFunctionAddressByOrdinalRoutine);

extern StpErr CallNativeLibraryFunction (void       *functionAddr,    // supplied
                                         void       *paramsOnStack,   // supplied: must be 16 bit aligned
                                         unsigned32 paramsSize,       // supplied: must be a multiple of 4
                                         unsigned32 *returnValueLow,  // returned: must be 16 bit aligned
                                         unsigned32 *returnValueHigh) // returned: must be 16 bit aligned or NULL
                      STYLETAP_TRAP(CallNativeLibraryFunctionRoutine);

extern StpErr UnloadNativeLibrary (unsigned32 libraryHandle) // supplied
                      STYLETAP_TRAP(UnloadNativeLibraryRoutine);

extern StpErr BringToForeground ()
                      STYLETAP_TRAP(BringToForegroundRoutine);

#define AcceptRegistrationKey   StpAcceptRegistrationKey

// StpAcceptRegistrationKey
//
// Purpose 
// - record a valid Registration Key on this device
//
// Returns
// - StpOK                  not previously registered, key is valid for this device, now registered
// - StpRegistered          previously registered, key not checked
// - StpParamError          not previously registered, key not syntactically valid or not valid for this device
// 
// Parameters
// - registrationKey (in)   null-terminated key string value. 
//                          May contain separator chars (" ./-"). Letters may be upper or lower case.   
extern StpErr StpAcceptRegistrationKey (const char *registrationKey) // supplied: null-terminated
                      STYLETAP_TRAP (StpAcceptRegistrationKeyRoutine);

#define GetProductKey           StpGetProductKey
extern StpErr StpGetProductKey (char   *productKey,             // returned: null-terminated
                             UInt16 *productKeyBufferSizeBytes) // supplied and returned: includes null terminator
                      STYLETAP_TRAP (StpGetProductKeyRoutine);

extern StpErr DatabaseFlush (DmOpenRef ref) // supplied
                      STYLETAP_TRAP (DatabaseFlushRoutine);

extern StpErr DatabaseFlushAll () 
                      STYLETAP_TRAP (DatabaseFlushAllRoutine);

extern StpErr GetMainWindowHandle (unsigned32 *handle) // returned
                      STYLETAP_TRAP (GetMainWindowHandleRoutine);

extern StpErr StpPPPConnect (unsigned32 *connectionHandle,	// returned
                             const char *parameters,
                             char       *errorBuffer,
                             unsigned16  errorBufferSize)
                      STYLETAP_TRAP (StpPPPConnectRoutine);

extern StpErr StpPPPHangUp (unsigned32 connectionHandle) // supplied
                      STYLETAP_TRAP (StpPPPHangUpRoutine);

// StpPPPConfig
//
// Purpose 
// - create or modify a Windows Mobile modem connection definition to be used to autodial an Internet connection
//
// Returns
// - StpOK
// - StpUnimplemented
// - StpParamError          (if required DialString parameter is missing)
// - StpUnspecifiedError
// 
// Parameters
// - parameters (in):      string containing name-value pairs for various connection settings
//                           ConnectionName (optional) default value is "DynamicPPPConnection"
//                           DialString     (required) include ALL of the digits that must be dialed by the modem
//                           UserName       (optional) default value is ""
//                           Password       (optional) default value is ""
//                           Domain         (optional) default value is ""
//                         The string must be formatted exactly as shown in the example below. 
//                         Insert your own values inside the quoted strings.
//                         Example: ConnectionName="cccccc" DialString="9999" UserName="uuuu" Password="pppp"
//                           
// - errorBuffer (out):    buffer where parsing errors, etc. are reported as a null-terminated string if the result is not StpOK  
// - errorBufferSize (in): size of the errorBuffer in bytes
// 
// Notes
// - The connection will use the modem called "Hayes Compatible on COM1:". The requested baud rate will be 115200.
// - The connection will be listed under the group "My ISP".
// - The connection will be used to autodial an Internet connection when a StyleTap application calls NetLibOpen if
//     - it is the only connection defined for "My ISP". This will be true after a hard reset if you do not create any other connections.
//     - under Settings/Connections/Advanced/Select Networks, the box labelled "Programs that automatically connect to the
//       Internet should connect using:" contains "My ISP". This is the default setting after a hard reset.
//
extern StpErr StpPPPConfig (const char *parameters,
                            char       *errorBuffer,
                            unsigned16  errorBufferSize)
                      STYLETAP_TRAP (StpPPPConfigRoutine);

extern StpErr StpExit (unsigned32 unusedParameter)	// supplied
					  STYLETAP_TRAP (StpExitRoutine);

// StpGetRegistrationState
//
// Purpose 
// - reports if StyleTap is registered and fully licensed on this device or running under a trial license 
// - if a trial license is in effect, reports the number of days remaining in the trial
//
// Returns
// - StpOK: check output parameters
// - StpUnimplemented
// - StpUnspecifiedError
// 
// Parameters
// - *registrationState (out):  StpRegistered or StpTrial
// - *trialDaysRemaining (out): if *registrationState == StpTrial, number of days remaining in trial
//
extern StpErr StpGetRegistrationState (StpErr     *registrationState,   // StpRegistered or StpTrial
                                       unsigned16 *trialDaysRemaining)  // returned (if registrationState == StpTrial)
                      STYLETAP_TRAP (StpGetRegistrationStateRoutine);

// StpGetRegistrationKeyFromServerExt
//
// Purpose
// - requests a StyleTap Registration Key from the StyleTap server for an established multiple license account
// - returns a new key for this device the first time it is called. Charges the account for one license.
// - returns the previously issued key for this device if it is called again. No charge to the account.
//
// Returns
// - StpOK: check output parameters (registrationKey and reissue)
// - StpUnimplemented
// - StpParamError
// - StpNoConnection
// - StpInvalidAccountIdPassword
// - StpInsufficientCredit
// - StpInvalidProductKey
// - StpTimeout
//
// Notes
// - requires a TCPIP connection. A preconfigured dialout connection can be established by calling NetLibOpen.
// - clientInfo is any null-terminated string supplied by the caller (max length = 255). You can use it to identify
//   this particular device (e.g. with an external serial number) or a batch of devices.
// - to obtain the Product Key for a device, call StpGetProductKey
// - to record the Registration Key on a device, call StpAcceptRegistrationKey with the value returned by 
//   StpGetRegistrationKeyFromServer
//
// Parameters
// - account (in):          your StyleTap registration server account id (supplied by StyleTap) (null-terminated string)
// - password (in):         password for your StyleTap server account (null-terminated string)
// - coupon (in):           optional coupon code (null-terminated string)
// - clientInfo (in):       any null terminated string (max length = 255)
// - productKey (in):       StyleTap Product Key for device being registered (null-terminated string)
// - registrationKey (out): (null-terminated string)
// - registrationKeyBufferSizeBytes (in): must be at least 25 chars to hold a formatted Registration Key
// - reissue (out):         set to 1 if this device was registered before (no charge to account) 
//                          or to 0 if this is a new registration (account has been charged)
// - timeoutInSystemTicks (in): maximum time allowed for transaction to complete
// - progress (in):             routine called periodically during the process of registration; 
//                              provides text suitable for display. Can be NULL if caller does not
//                              want to provide a progress routine. Also allows process to be
//                              prematurely terminated if a non-zero value returned.
// - userData (in):             arbitrary pointer passed through to progress routine.


typedef unsigned8 /* non-zero to abort */ (*GetRegKeyProgressProc) (const char *progressText, void *userData);

extern StpErr StpGetRegistrationKeyFromServerExt 
(const char            *account,                        // supplied
 const char            *password,                       // supplied
 const char            *coupon,                         // supplied
 const char            *clientInfo,                     // supplied
 const char            *productKey,                     // supplied 
 char                  *registrationKey,                // null-terminated string returned
 unsigned16            registrationKeyBufferSizeBytes,  // supplied
 unsigned32            *reissue,                        // returned
 signed32              timeoutInSystemTicks,            // supplied
 GetRegKeyProgressProc progress,                        // supplied
 void                  *userData                        // supplied
)
STYLETAP_TRAP (StpGetRegistrationKeyFromServerRoutineExt);

// StpRefreshDisplayWindow
//
// Purpose 
// - ensures that the current display buffer contents appear on the display of the Windows device and forces the screen back
//   into the active state 
//
// Returns
// - StpOK
//
// Parameters
// - none
//
extern StpErr StpRefreshDisplayWindow ()
                      STYLETAP_TRAP (StpRefreshDisplayWindowRoutine);

// return non-zero to cancel processing
typedef unsigned8 (*PostGetCallback) (unsigned32 bytesProcessed, void *userData);

extern StpErr StpPostDatabase (unsigned16       cardNo, 
                               unsigned32       dbID, 
                               const char       *host, 
                               unsigned16       port,
                               const char       *uri,
                               unsigned8        ssl, 
                               signed32         timeout,
                               PostGetCallback  callback,
                               void             *callbackUserData,
                               unsigned32       *httpResponseCode)
                           STYLETAP_TRAP (StpPostDatabaseRoutine);

typedef unsigned8 (*deleteProcType) (const char *name, unsigned16 version, unsigned16 cardNo, 
                                     unsigned32 dbID, void *userData);

extern StpErr StpGetDatabase (unsigned16         cardNo,
                              const char         *host, 
                              unsigned16         port,
                              const char         *uri,
                              unsigned8          ssl, 
                              signed32           timeout,
                              PostGetCallback    callback,
                              void               *callbackUserData,
                              unsigned32         *httpResponseCode,
                              deleteProcType     deleteProc,
                              void               *userData,
                              unsigned32         *dbID)
                           STYLETAP_TRAP (StpGetDatabaseRoutine);

// if toPort == 0 then the fromPort is no longer remapped
extern StpErr StpSetSerialPortMapping (unsigned32 fromPort, unsigned32 toPort)
                  STYLETAP_TRAP (StpSetSerialPortMappingRoutine);

// if remapped == false then the default mapping is returned
// if there is no mapping (default or override) then toPort == 0
extern StpErr StpGetSerialPortMapping (unsigned8  remapped, // 0 = default mapping, !0 is override remapping 
                                       unsigned32 fromPort, 
                                       unsigned32 *toPort)
                  STYLETAP_TRAP (StpGetSerialPortMappingRoutine);

// StpSetFastScreenMode
//
// Purpose 
// - enables or disables use of GAPI for internal screen handling. (GAPI on = "Fast screen mode") 
//
// Returns
// - StpOK
//
// Notes
// - if the SIP is up and overlaps the application portion of the screen,
//   GAPI is always off. In that case, you can enable GAPI with this API,
//   but it will not take effect until the SIP is dismissed.
//
// Parameters
// - enable (in):        1 = GAPI/Fast screen mode on
//                       0 = GAPI/Fast screen mode off
// - previousMode (out): mode (1, 0) that was in effect before this call
//
extern StpErr StpSetFastScreenMode (unsigned8    enable,
                                    unsigned8    *previousMode)
                  STYLETAP_TRAP (StpSetFastScreenModeRoutine);


// StpSetSipScreenPosition
//
// Purpose 
// - enables or disables vertical shifting of application display area
//   when the SIP keyboard is displayed
//
// Returns
// - StpOK
//
// Parameters
// - SIPShiftMode (in):  0 = When the SIP keyboard is displayed, hide the 
//                           Windows navigation bar at the top and shift 
//                           the StyleTap application display area up, 
//                           to show as much of the application as 
//                           possible. (Normal, default behavior.)
//                       1 = Do not hide the navigation bar at the top and 
//                           do not shift the application display area when
//                           the SIP keyboard is displayed.
// - previousMode (out): mode (1 or 0) that was in effect before this call
//
extern StpErr StpSetSipScreenPosition (SipScreenPositionType   SIPShiftMode,
                                       SipScreenPositionType   *previousMode)
                  STYLETAP_TRAP (StpSetSipScreenPositionRoutine);


// StpUnloadDatabase
// 
// Purpose
// - immediately frees all memory that has been allocated for a database 
//   that is now closed
//
// Returns
// - StpOK 
// - StpDbNotFound              dbID not valid
// - StpRecordDbRequired        recordDbOnly is true and dbID refers to a 
//                              resource database (no memory freed)
// - StpDbOpen                  open count for dbID is not zero (no memory 
//                              freed)
// - StpLockError               freeLockedChunks is false and some chunks 
//                              are locked (no memory freed)
//
// Parameters
// - dbId                       as returned from DmFindDatabase
// - recordDbOnly               1 (true)  ==> call fails if dbId refers to 
//                                            a resource database
//                              0 (false) ==> dbId can refer to a resource 
//                                            or a record database
// - freeLockedChunks           1 (true)  ==> ignore locks; free all 
//                                            associated memory. Dangerous, 
//                                            especially for resource db.
//                              0 (false) ==> do not free any memory if any 
//                                            chunk is locked
//
extern StpErr StpUnloadDatabase (unsigned32 dbID,
                                 unsigned8  recordDbOnly,
                                 unsigned8  freeLockedChunks)
                  STYLETAP_TRAP (StpUnloadDatabaseRoutine);


// StpUnloadDatabases
//
// Purpose
// - immediately frees all memory that has been allocated for all databases 
//   that are now closed
//
// Returns
// - StpOK
//
// Parameters                   none
// - recordDbOnly               1 (true)  ==> unloads all closed record 
//                                            databases; ignores resource 
//                                            databases
//                              0 (false) ==> unloads all closed record  
//                                            and resource databases
// - freeLockedChunks           1 (true)  ==> ignore locks; free all 
//                                            possible memory. Dangerous, 
//                                            especially for resource db.
//                              0 (false) ==> do not free any memory for a 
//                                            database if any chunk in 
//                                            that database is locked
//
extern StpErr StpUnloadDatabases (unsigned8  recordDbOnly,
                                  unsigned8  freeLockedChunks)
                  STYLETAP_TRAP (StpUnloadDatabasesRoutine);

// StpUpdateDisplayBuffer
//
// Purpose 
// - used by application to notify StyleTap that it has written directly to the a screen from within an ARMlet 
// - no effect if window is not active
//
// Returns
// - StpOK
//
// Parameters
// - none
//
extern StpErr StpUpdateDisplayBuffer ()
                      STYLETAP_TRAP (StpUpdateDisplayBufferRoutine);

// StpSpecialEffect
//
// Purpose 
// - used by application to cause special effect (if supported by device) 
//
// Returns
// - StpOK
// - StpNotSupportedByDevice
// - StpParamError
// - StpUnspecifiedError 
// - StpOutOfMemory
//
// Parameters
// - type                       type of special effect
//                                 ledSpecialEffect
//                                 vibrateSpecialEffect
// - count                      number of times to repeat special effect. 
//                                0xffffffff - turns on special effect indefinitely until called again passing 0 for this parm.            
//                                0          - stop special effect  
// - onDuration                 duration of each special effect in centiseconds. Ignored if count is 0xffffffff or 0. Should be non-zero otherwise.
// - offDuration                duration between special effects in centiseconds. Ignored if count is 0xffffffff or 0.
//
// 
extern StpErr StpSpecialEffect (unsigned8  type,
								unsigned32 count,
						        unsigned32 onDuration,
						        unsigned32 offDuration)
                      STYLETAP_TRAP (StpSpecialEffectRoutine);

// StpGetObexReceiverInstallStatus
//
// Purpose 
// - used by application to find out whether StyleTap Obex Receiver DLL is installed 
//
// Returns
// - StpOK
// - StpUnspecifiedError 
//
// Parameters
// - installed (out)           0 - not installed
//                             non-zero installed
//
// 
extern StpErr StpGetObexReceiverInstallStatus (unsigned8 *  installed)
                      STYLETAP_TRAP (StpGetObexReceiverInstallStatusRoutine);

// StpSetObexReceiverInstallStatus
//
// Purpose 
// - used by application to install/uninstall StyleTap Obex Receiver DLL 
//
// Returns
// - StpOK
// - StpUnspecifiedError 
//
// Parameters
// - install (in)             0 - uninstall
//                            1 - install
//
// 
extern StpErr StpSetObexReceiverInstallStatus (unsigned8   install)
                      STYLETAP_TRAP (StpSetObexReceiverInstallStatusRoutine);

/////
//
// encodingType values:
//
//   3 - default Latin
//  47 - ISSO 8859-9 (Turkish)

extern StpErr StpIsCharEncodingAvailable (unsigned8 encodingType)
                      STYLETAP_TRAP (StpIsCharEncodingAvailableRoutine);

extern StpErr StpSetCurrentCharEncoding (unsigned8 encodingType)
                      STYLETAP_TRAP (StpSetCurrentCharEncodingRoutine);

extern StpErr StpGetCurrentCharEncoding (unsigned8 *encodingType)
                      STYLETAP_TRAP (StpGetCurrentCharEncodingRoutine);

extern StpErr StpSetSimulateDoubleDensityScreen (unsigned8 setting)
                      STYLETAP_TRAP (StpSetSimulateDoubleDensityScreenRoutine);

extern StpErr StpGetSimulateDoubleDensityScreen (unsigned8 *setting)
                      STYLETAP_TRAP (StpGetSimulateDoubleDensityScreenRoutine);


#ifdef __cplusplus
}
#endif

#endif
